﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ChaserController : MonoBehaviour
{
    enum State { EARLY, LATE, BLOCKED }; // possible states

    float speed;
    State state;

    float vertOffset = 0f;
    float maxSpeed = 28;
    float acceleration = 20;
    float angularAcceleration = 5;

    RunnerLocator locator;
    Vector3 currentPosition;
    int turnAmount;

    GameObject ballRunner;
    RunnerController runnerController;

    void Start()
    {
        ballRunner = GameObject.Find("Ball Runner");
        runnerController = ballRunner.GetComponent<RunnerController>();
        speed = 0;
    }

    public void Initialize(Platform p, RunnerLocator loc)
    {
        locator = new RunnerLocator(p, loc.Row, loc.Col, loc.Direction);
        currentPosition = CurrentCenterPosition();
        speed = 0;
        state = State.LATE;
    }

    Vector3 CurrentCenterPosition()
    {
        return locator.CenterPoint() + Vector3.up * vertOffset;
    }

    void UpdateSpeed(float deltaTime)
    {
        float speedIncrease = acceleration * deltaTime;
        speed += speedIncrease;
        if (speed > maxSpeed)
        {
            speed = maxSpeed;
        }
    }

    void UpdateState(State newState)
    {
        state = newState;
        turnAmount = 0;
    }


    // The update procedure is a close copy of the runner's update, but whenever decisions are needed
    // we extract them from the queue.
    void Update()
    {
        UpdateSpeed(Time.deltaTime);
        Vector3 idealPosition = currentPosition + locator.DirectionVector() * speed * Time.deltaTime;
        if (state == State.EARLY)
        {
            if (locator.IsEarly(idealPosition)) // still early
            {
                currentPosition = idealPosition;
            }
            else // just passed the midpoint
            {
                if (runnerController.TurnSequenceIsEmpty()) // this shouldn't happen!
                {
                    turnAmount = 0;
                }
                else // get the next turn from the queue
                {
                    turnAmount = runnerController.DequeueFromTurnSequence();
                }

                if (turnAmount >= 0) // valid turn or continuation
                {
                    if (turnAmount != 0) // nontrivial turn?
                    {
                        locator.MakeTurn(turnAmount);
                        currentPosition = CurrentCenterPosition();
                    }
                    else // continue through midpoint
                    {
                        currentPosition = idealPosition;
                    }
                    UpdateState(State.LATE);
                }
                else // invalid turn or cannot continue
                {
                    currentPosition = CurrentCenterPosition();
                    UpdateState(State.BLOCKED);
                }
            }
        }
        else if (state == State.LATE)
        {
            if (!locator.InSameCell(currentPosition, idealPosition))
            {
                locator.UpdateFromNewPosition(idealPosition);
                UpdateState(State.EARLY);
            }
            currentPosition = idealPosition;
        }
        else // (state == State.BLOCKED)
        {
            if (!runnerController.TurnSequenceIsEmpty())
            {
                turnAmount = runnerController.DequeueFromTurnSequence();
                locator.MakeTurn(turnAmount);
                UpdateState(State.LATE);
            }
        }

        transform.position = currentPosition;

        // adjust direction to turn towards the runner
        Vector3 towardsRunner = runnerController.Position - transform.position;
        towardsRunner.y = 0; // ignore up/down component
        Vector3 newDir = Vector3.RotateTowards(transform.forward, towardsRunner, angularAcceleration * Time.deltaTime, 0.0f);
        transform.rotation = Quaternion.LookRotation(newDir);
    }
}
