﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class RunnerController : MonoBehaviour {

    enum State { BEFORE, BEYOND, BLOCKED }; // possible states

    float speed; // current speed
    State state; // current state

    float vertOffset = 1.5f;
    float maxSpeed = 30;
    float acceleration = 20;
    float speedPenalty = 0.25f;

    RunnerLocator locator; // locator indicates the tile we are in
    RunnerLocator targetLocator; // locator for the target tile
    Vector3 currentPosition; // coordinates of our current position
    public bool turnRequested; // true if a turn has been requested
    public int turnAmount; // turn requested relative to current direction (0 = continue straight)

    Queue<int> turnSequence; // saved turn sequences for use by the chaser

    GameController gameController;

    void Start()
    {
        gameController = GameObject.Find("Game Controller").GetComponent<GameController>();
    }

    public void Initialize(Platform p, RunnerLocator loc, RunnerLocator target)
    {
        locator = new RunnerLocator(p, loc.Row, loc.Col, loc.Direction);
        targetLocator = new RunnerLocator(p, target.Row, target.Col, 0);
        p.MakeTarget(target.Row, target.Col);
        currentPosition = CurrentCenterPosition();
        speed = 0;
        state = State.BEYOND;
        turnRequested = false;
        turnAmount = 0;
        turnSequence = new Queue<int>();
        turnSequence.Enqueue(0); // add an artificial starting turn
    }

    Vector3 CurrentCenterPosition()
    {
        return locator.CenterPoint() + Vector3.up * vertOffset;
    }

    public Vector3 Position
    {
        get { return currentPosition; }
    }

    public Vector3 EulerAngles
    {
        get { return locator.EulerAngles(); }
    }

    public void RequestTurn(int t)
    {
        if ((state == State.BLOCKED || locator.IsEarly(currentPosition)) && locator.CanTurn(t))
        {
            turnRequested = true;
            turnAmount = t;
        }
        else
        {
            speed *= speedPenalty;
        }
    }

    void UpdateSpeed(float deltaTime)
    {
         if (state != State.BLOCKED && speed < maxSpeed)
        {
            float speedIncrease = acceleration * deltaTime;
            speed += speedIncrease;
            if (speed > maxSpeed)
            {
                speed = maxSpeed;
            }
        }
    }

    void UpdateState(State newState)
    {
        state = newState;
        turnRequested = false;
        turnAmount = 0;
    }

    public bool TurnSequenceIsEmpty()
    {
        return turnSequence.Count == 0;
    }

    public int DequeueFromTurnSequence()
    {
        return turnSequence.Dequeue();
    }

    // Warning: The following procedure assumes that displacements are always smaller than half cell size

    void Update()
    {
        UpdateSpeed(Time.deltaTime);
        Vector3 idealPosition = currentPosition + locator.DirectionVector() * speed * Time.deltaTime;
        if (state == State.BEFORE)
        {
            if (locator.IsEarly(idealPosition)) // still early
            {
                currentPosition = idealPosition;
            }
            else // just passed the midpoint
            {
                if (locator.SameCell(targetLocator))
                {
                    gameController.SignalWin();
                }
                if (locator.CanTurn(turnAmount)) // valid turn or continuation
                {
                    if (turnAmount != 0) // nontrivial turn?
                    {
                        locator.MakeTurn(turnAmount);
                        currentPosition = CurrentCenterPosition();
                    }
                    else // continue through midpoint
                    {
                        currentPosition = idealPosition;
                    }
                    turnSequence.Enqueue(turnAmount);
                    UpdateState(State.BEYOND);
                }
                else // invalid turn or cannot continue
                {
                    currentPosition = CurrentCenterPosition();
                    UpdateState(State.BLOCKED);
                    speed = 0;
                }
            }
        }
        else if (state == State.BEYOND)
        {
           if (!locator.InSameCell(currentPosition, idealPosition))
            {
                locator.UpdateFromNewPosition(idealPosition);
                UpdateState(State.BEFORE);
            }
            currentPosition = idealPosition;
        }
        else // (state == State.BLOCKED)
        {
            if (turnRequested)
            {
                if (locator.CanTurn(turnAmount))
                {
                    locator.MakeTurn(turnAmount);
                    turnSequence.Enqueue(turnAmount);
                    UpdateState(State.BEYOND);
                }
            }
        }
        transform.position = currentPosition;
    }

    void OnTriggerEnter(Collider other)
    {
        if (other.gameObject.CompareTag("BunnyPin"))
        {
            gameController.SignalLoss();
        }

    }
}
