﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

// This class encapsulates a tile and a direction

public class RunnerLocator  {

    Platform thePlatform;

    int row;
    int col;
    int direction; // 0, ..., 3 = N, E, S, W; -1 = Stationary

    static int[] zOffset = { 0, +1, 0, -1, 0 }; // directional offsets for rows (z)
    static int[] xOffset = { 0, 0, +1, 0, -1 }; // directional offsets for columns (x)

    public RunnerLocator(Platform p, int r, int c, int d)
    {
        thePlatform = p;
        row = r;
        col = c;
        direction = d;
    }

    public int Row
    {
        get { return row; }
    }

    public int Col
    {
        get { return col; }
    }

    public int Direction
    {
        get { return direction; }
    }

    public RunnerLocator Neighbor(int d)
    {
        return new RunnerLocator(thePlatform, row + zOffset[(direction + d) % 4 + 1], col + xOffset[(direction + d) % 4 + 1], direction);
    }

    public Vector3 CenterPoint()
    {
        return thePlatform.CenterPoint(row, col);
    }

    public Vector3 DirectionVector()
    {
        return new Vector3(xOffset[direction + 1], 0, zOffset[direction + 1]);
    }

    public Vector3 EulerAngles()
    {
        return new Vector3(0, Mathf.Max(0, direction) * 90, 0);
    }

    public bool Exists()
    {
        return thePlatform.Exists(row, col);
    }

    public void MakeTurn(int turn)
    {
        direction = (direction + turn) % 4;
    }

    public bool CanTurn(int turn)
    {
        int newDirection = (direction + turn) % 4;
        return thePlatform.Exists(row + zOffset[newDirection + 1], col + xOffset[newDirection + 1]);
    }

    public bool CanContinue()
    {
        return CanTurn(0);
    }

    public void UpdateFromNewPosition(Vector3 position)
    {
        row = thePlatform.RowIndex(position.z);
        col = thePlatform.ColIndex(position.x);
    }

    public bool SameCell(RunnerLocator other)
    {
        return (other.row == row && other.col == col);
    }

    public bool InSameCell(Vector3 pos1, Vector3 pos2)
    {
        return (thePlatform.RowIndex(pos1.z) == thePlatform.RowIndex(pos2.z) && 
                thePlatform.ColIndex(pos1.x) == thePlatform.ColIndex(pos2.x));
    }

    public bool IsEarly(Vector3 pos)
    {
        Vector3 center = CenterPoint();
        switch (direction)
        {
            case 0: return pos.z < center.z;
            case 1: return pos.x < center.x;
            case 2: return pos.z > center.z;
            case 3: return pos.x > center.x;
            default: return false;
        }
    }

}
